package com.isu.gaswellwatch.modbus.data;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.SuperBuilder;

import java.io.Serial;
import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

/**
 * 网关标识/设备标识/命令标识/采集下发时间/采集数据接收时间/采集数据体
 * 4B454E454E4731343030303030333538/1/10000/1732365690000/1732365691000/01042C07D000010003000C002B0029000300010000001A0018003303CD002300080DAC0DAC0000000000000959091DF066
 *
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 10:11
 */
@Getter
@Setter
@SuperBuilder
@NoArgsConstructor
public class ModbusMessage implements Serializable {
    @Serial
    private static final long serialVersionUID = 429189115651027028L;
    /**
     * 队列地址
     */
    private String queueName;
    /**
     * 网关标识
     */
    private String gatewayIdentifier;
    /**
     * 设备标识
     */
    private Long deviceId;
    /**
     * 指令标识
     */
    private Long commandId;
    /**
     * 采集指令下发时间
     */
    private Long collectionTime;
    /**
     * 采集数据接收时间
     */
    private Long receiveTime;
    /**
     * 采集原始字符串
     */
    private String message;
    /**
     * key: 点位地址, value: 点位解析值
     */
    private Map<String, MessagePoint> messagePointMap;

    /**
     * 点位解析数据
     */
    @Getter
    @Setter
    @SuperBuilder
    @NoArgsConstructor
    public static class MessagePoint implements Serializable {
        @Serial
        private static final long serialVersionUID = -3118301653064611676L;
        /**
         * 原始解析值
         */
        private String originalValue;
        /**
         * 初始解析值
         */
        private String parseValue;
        /**
         * 修正解析值
         */
        private String value;

        public String getValue() {
            return Objects.isNull(this.value) ? this.parseValue : this.value;
        }
    }
}
